@@ -162,4 +162,11 @@ sub getback {
 
 ###############################################################################
 ###############################################################################
+
+=head1 AUTHOR
+
+Kevin Cody-Little <kcody@cpan.org>
+
+=cut
+
 1;
