@@ -68,6 +68,10 @@ Retrieves a human-redable string identifying the variable.
 
 =head1 OVERRIDDES
 
+=head2 $this->remove
+
+Overridden from IPC::Shm::Simple to remove table entries for named variables.
+
 =head2 $this->DETACH
 
 Called by IPC::Shm::Simple when the last in-process instance
@@ -244,18 +248,42 @@ sub vartype {
 }
 
 
+###############################################################################
+# deliberate removal override
+
+sub remove {
+	my ( $this ) = @_;
+
+	if ( my $vname = $this->varname ) {
+		delete $IPC::Shm::NAMEVARS{$vname};
+		$this->decref;
+		$this->CLEAR;
+	}
+
+	return $this->SUPER::remove();
+}
+
+
 ###############################################################################
 # disconnect-time cleanups
 
 sub DETACH {
 	my ( $this ) = @_;
 
-	unless ( $this->nrefs ) {
-		$this->CLEAR;
-		$this->remove;
-		if ( my $vanon = $this->varanon ) {
+	unless ( $this->nrefs or $this->varname ) {
+
+		my $vanon = $this->varanon;
+
+		$this->writelock;
+
+		if ( $this->nconns == 1 ) {
+
+			$this->CLEAR;
+			$this->SUPER::remove;
+
 			delete $IPC::Shm::ANONVARS{$vanon};
 			delete $IPC::Shm::ANONTYPE{$vanon};
+
 		}
 
 	}
